@@ -15,9 +15,12 @@ Contains :class:`SpecificationEditorWindow`.
 :date:   11.12.2020
 """
 from copy import deepcopy
-from PySide2.QtCore import QItemSelectionModel, QModelIndex, Qt, Signal, Slot
+import json
+from PySide2.QtCore import QItemSelectionModel, QMimeData, QModelIndex, QPoint, Qt, Signal, Slot
 from PySide2.QtGui import QKeySequence
-from PySide2.QtWidgets import QAction, QHeaderView
+from PySide2.QtWidgets import QAction, QApplication, QHeaderView, QMenu
+
+from spinedb_api.mapping import unflatten
 from spinedb_api.export_mapping import (
     alternative_export,
     feature_export,
@@ -28,6 +31,8 @@ from spinedb_api.export_mapping import (
     object_parameter_export,
     parameter_value_list_export,
     relationship_export,
+    relationship_object_parameter_default_value_export,
+    relationship_object_parameter_export,
     relationship_parameter_default_value_export,
     relationship_parameter_export,
     scenario_alternative_export,
@@ -46,9 +51,9 @@ from spinedb_api.export_mapping.group_functions import (
     GROUP_FUNCTION_DISPLAY_NAMES,
     group_function_name_from_display,
     group_function_display_from_name,
+    NoGroup,
 )
 from spinetoolbox.project_item.specification_editor_window import SpecificationEditorWindowBase
-from ...commands import RenameMapping
 from .preview_updater import PreviewUpdater
 from ..commands import (
     ChangeWriteOrder,
@@ -57,6 +62,7 @@ from ..commands import (
     EnableAllMappings,
     NewMapping,
     RemoveMapping,
+    RenameMapping,
     SetAlwaysExportHeader,
     SetMappingEnabled,
     SetMappingType,
@@ -64,6 +70,8 @@ from ..commands import (
     SetMapping,
     SetUseFixedTableNameFlag,
     SetFixedTableName,
+    SetGroupFunction,
+    SetHighlightDimension,
 )
 from ..mvcmodels.mappings_table_model import MappingsTableModel
 from ..mvcmodels.mapping_editor_table_model import EditorColumn, MappingEditorTableModel
@@ -83,6 +91,8 @@ mapping_type_to_combo_box_label = {
     MappingType.object_parameter_values: "Object class",
     MappingType.parameter_value_lists: "Parameter value list",
     MappingType.relationships: "Relationship class",
+    MappingType.relationship_object_parameter_default_values: "Relationship class with object parameter",
+    MappingType.relationship_object_parameter_values: "Relationship class with object parameter",
     MappingType.relationship_parameter_default_values: "Relationship class",
     MappingType.relationship_parameter_values: "Relationship class",
     MappingType.scenario_alternatives: "Scenario alternative",
@@ -102,6 +112,8 @@ mapping_type_to_parameter_type_label = {
     MappingType.object_parameter_values: "Value",
     MappingType.parameter_value_lists: "None",
     MappingType.relationships: "None",
+    MappingType.relationship_object_parameter_default_values: "Default value",
+    MappingType.relationship_object_parameter_values: "Value",
     MappingType.relationship_parameter_default_values: "Default value",
     MappingType.relationship_parameter_values: "Value",
     MappingType.scenario_alternatives: "None",
@@ -112,6 +124,9 @@ mapping_type_to_parameter_type_label = {
 }
 
 
+_MAPPINGS_MIME_TYPE = "application/spine_items-exportmappings"
+
+
 class SpecificationEditorWindow(SpecificationEditorWindowBase):
     """Interface to edit exporter specifications."""
 
@@ -154,6 +169,16 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._remove_mapping_action.setShortcut(QKeySequence(QKeySequence.Delete))
         self._remove_mapping_action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         self._remove_mapping_action.triggered.connect(self._delete_mapping)
+        self._copy_mappings_action = QAction("Copy", self)
+        self._copy_mappings_action.setShortcut(QKeySequence(QKeySequence.Copy))
+        self._copy_mappings_action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+        self._copy_mappings_action.triggered.connect(self._copy_mappings_to_clipboard)
+        self._paste_mappings_action = QAction("Paste", self)
+        self._paste_mappings_action.setShortcut(QKeySequence(QKeySequence.Paste))
+        self._paste_mappings_action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
+        self._paste_mappings_action.triggered.connect(self._paste_mappings_from_clipboard)
+        self._duplicate_mappings_action = QAction("Duplicate", self)
+        self._duplicate_mappings_action.triggered.connect(self._duplicate_mappings)
         self._ui.remove_mapping_button.clicked.connect(self._remove_mapping_action.trigger)
         self._toggle_all_enabled_action = QAction("Toggle all mappings enabled", self)
         self._toggle_all_enabled_action.triggered.connect(self._toggle_all_enabled)
@@ -164,11 +189,15 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._write_later_action = QAction("Write later", self)
         self._write_later_action.triggered.connect(self._write_later)
         self._ui.write_later_button.clicked.connect(self._write_later_action.trigger)
+        self._mappings_table_menu = self._make_mappings_table_menu()
         self._ui.mappings_table.addAction(self._add_mapping_action)
         self._ui.mappings_table.addAction(self._remove_mapping_action)
+        self._ui.mappings_table.addAction(self._copy_mappings_action)
+        self._ui.mappings_table.addAction(self._paste_mappings_action)
         self._ui.mappings_table.setModel(self._sort_mappings_table_model)
         self._ui.mappings_table.setSortingEnabled(True)
         self._ui.mappings_table.sortByColumn(0, Qt.AscendingOrder)
+        self._ui.mappings_table.customContextMenuRequested.connect(self._show_mappings_table_context_menu)
         self._expect_current_mapping_change(False)
         self._ui.mappings_table.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
         self._ui.group_fn_combo_box.addItems(GROUP_FUNCTION_DISPLAY_NAMES)
@@ -177,6 +206,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._ui.parameter_dimensions_spin_box.valueChanged.connect(self._change_parameter_dimensions)
         self._ui.always_export_header_check_box.stateChanged.connect(self._change_always_export_header)
         self._ui.relationship_dimensions_spin_box.valueChanged.connect(self._change_relationship_dimensions)
+        self._ui.highlight_dimension_spin_box.valueChanged.connect(self._change_highlight_dimension)
         self._ui.fix_table_name_check_box.stateChanged.connect(self._change_fix_table_name_flag)
         self._ui.fix_table_name_line_edit.editingFinished.connect(self._change_fix_table_name)
         self._ui.group_fn_combo_box.currentTextChanged.connect(self._change_root_mapping_group_fn)
@@ -320,27 +350,41 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._set_mapping_type_silently(mapping_type_to_combo_box_label[mapping_type])
         self._set_parameter_type_silently(mapping_type_to_parameter_type_label[mapping_type])
         self._set_always_export_header_silently(current.data(MappingsTableModel.ALWAYS_EXPORT_HEADER_ROLE))
-        if mapping_type in (
+        if mapping_type in {
             MappingType.relationships,
             MappingType.relationship_parameter_values,
             MappingType.relationship_parameter_default_values,
-        ):
+            MappingType.relationship_object_parameter_values,
+            MappingType.relationship_object_parameter_default_values,
+        }:
             self._set_relationship_dimensions_silently(current.data(MappingsTableModel.RELATIONSHIP_DIMENSIONS_ROLE))
         else:
             self._set_relationship_dimensions_silently(1)
-        if mapping_type in (
+        if mapping_type in {
+            MappingType.relationship_object_parameter_values,
+            MappingType.relationship_object_parameter_default_values,
+        }:
+            self._set_highlight_dimension_silently(current.data(MappingsTableModel.HIGHLIGHT_DIMENSION_ROLE))
+            self._ui.highlight_dimension_spin_box.setMaximum(
+                current.data(MappingsTableModel.RELATIONSHIP_DIMENSIONS_ROLE)
+            )
+        else:
+            self._set_highlight_dimension_silently(0)
+        if mapping_type in {
             MappingType.object_parameter_values,
             MappingType.object_parameter_default_values,
             MappingType.relationship_parameter_values,
             MappingType.relationship_parameter_default_values,
-        ):
+            MappingType.relationship_object_parameter_values,
+            MappingType.relationship_object_parameter_default_values,
+        }:
             self._set_parameter_dimensions_silently(current.data(MappingsTableModel.PARAMETER_DIMENSIONS_ROLE))
         else:
             self._set_parameter_dimensions_silently(0)
         self._set_use_fixed_table_name_flag_silently(current.data(MappingsTableModel.USE_FIXED_TABLE_NAME_FLAG_ROLE))
         self._set_fixed_table_name_silently(current.data(MappingsTableModel.FIXED_TABLE_NAME_ROLE))
+        self._set_group_fn_silently(group_function_display_from_name(current.data(MappingsTableModel.GROUP_FN_ROLE)))
         root_mapping = current.data(MappingsTableModel.MAPPING_ROOT_ROLE)
-        self._set_root_mapping_group_fn_silently(group_function_display_from_name(root_mapping.group_fn))
         mapping_name = self._mappings_table_model.index(current.row(), 0).data()
         self._mapping_editor_model.set_mapping(mapping_name, root_mapping)
         self._enable_relationship_controls()
@@ -378,16 +422,18 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         if sorted_index != self._ui.mappings_table.currentIndex():
             self._ui.mappings_table.selectionModel().setCurrentIndex(sorted_index, QItemSelectionModel.ClearAndSelect)
             return
+        enable_controls = False
         if MappingsTableModel.MAPPING_ROOT_ROLE in roles:
             root_mapping = top_left.data(MappingsTableModel.MAPPING_ROOT_ROLE)
             self._mapping_editor_model.set_mapping(top_left.data(Qt.DisplayRole), root_mapping)
             self._set_relationship_dimensions_silently(top_left.data(MappingsTableModel.RELATIONSHIP_DIMENSIONS_ROLE))
             self._set_parameter_dimensions_silently(top_left.data(MappingsTableModel.PARAMETER_DIMENSIONS_ROLE))
-            self._set_root_mapping_group_fn_silently(group_function_display_from_name(root_mapping.group_fn))
+            enable_controls = True
         if MappingsTableModel.MAPPING_TYPE_ROLE in roles:
             mapping_type = top_left.data(MappingsTableModel.MAPPING_TYPE_ROLE)
             self._set_mapping_type_silently(mapping_type_to_combo_box_label[mapping_type])
             self._set_parameter_type_silently(mapping_type_to_parameter_type_label[mapping_type])
+            enable_controls = True
         if MappingsTableModel.ALWAYS_EXPORT_HEADER_ROLE in roles:
             self._set_always_export_header_silently(top_left.data(MappingsTableModel.ALWAYS_EXPORT_HEADER_ROLE))
         if MappingsTableModel.USE_FIXED_TABLE_NAME_FLAG_ROLE in roles:
@@ -396,8 +442,38 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             )
         if {MappingsTableModel.MAPPING_ROOT_ROLE, MappingsTableModel.FIXED_TABLE_NAME_ROLE} & set(roles):
             self._set_fixed_table_name_silently(top_left.data(MappingsTableModel.FIXED_TABLE_NAME_ROLE))
-        self._enable_relationship_controls()
-        self._enable_parameter_controls()
+        if MappingsTableModel.GROUP_FN_ROLE in roles:
+            self._set_group_fn_silently(
+                group_function_display_from_name(top_left.data(MappingsTableModel.GROUP_FN_ROLE))
+            )
+        if MappingsTableModel.HIGHLIGHT_DIMENSION_ROLE in roles:
+            self._set_highlight_dimension_silently(top_left.data(MappingsTableModel.HIGHLIGHT_DIMENSION_ROLE))
+        if enable_controls:
+            self._enable_relationship_controls()
+            self._enable_parameter_controls()
+
+    @Slot(QPoint)
+    def _show_mappings_table_context_menu(self, position):
+        """Shows Mappings table context menu.
+
+        Args:
+            position (QPoint): requested menu position
+        """
+        self._mappings_table_menu.exec_(self._ui.mappings_table.mapToGlobal(position))
+
+    def _make_mappings_table_menu(self):
+        """Creates context menu for Mappings table.
+
+        Returns:
+            QMenu: context menu
+        """
+        menu = QMenu(self)
+        menu.addAction(self._add_mapping_action)
+        menu.addAction(self._copy_mappings_action)
+        menu.addAction(self._paste_mappings_action)
+        menu.addAction(self._duplicate_mappings_action)
+        menu.addAction(self._remove_mapping_action)
+        return menu
 
     @Slot(bool)
     def _new_mapping(self, _=True):
@@ -443,6 +519,75 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
                 self._undo_stack.push(RemoveMapping(row, self._mappings_table_model))
             self._undo_stack.endMacro()
 
+    @Slot(bool)
+    def _copy_mappings_to_clipboard(self, checked):
+        """Copies selected mappings to system clipboard.
+
+        Args:
+            checked (bool): unused
+        """
+        selection_model = self._ui.mappings_table.selectionModel()
+        indices = [self._sort_mappings_table_model.mapToSource(i) for i in selection_model.selectedIndexes()]
+        if not indices:
+            return
+        specifications = []
+        for index in indices:
+            if index.column() != 0:
+                continue
+            specification = self._mappings_table_model.data(index, MappingsTableModel.MAPPING_SPECIFICATION_ROLE)
+            serialized = specification.to_dict()
+            serialized["name"] = self._mappings_table_model.data(index)
+            specifications.append(serialized)
+        clipboard = QApplication.clipboard()
+        mime_data = QMimeData()
+        mime_data.setData(_MAPPINGS_MIME_TYPE, bytes(json.dumps(specifications), "utf-8"))
+        clipboard.setMimeData(mime_data)
+
+    @Slot(bool)
+    def _paste_mappings_from_clipboard(self, checked):
+        """Pastes mappings from system clipboard.
+
+        Args:
+            checked (bool): unused
+        """
+        clipboard = QApplication.clipboard()
+        mime_data = clipboard.mimeData()
+        if not mime_data.hasFormat(_MAPPINGS_MIME_TYPE):
+            return
+        serialized = json.loads(str(mime_data.data(_MAPPINGS_MIME_TYPE), "utf-8"))
+        names = [d.pop("name") for d in serialized]
+        mappings = [MappingSpecification.from_dict(d) for d in serialized]
+        command_text = "paste mapping" + ("s" if len(mappings) > 1 else "")
+        self._undo_stack.beginMacro(command_text)
+        for mapping, name in zip(mappings, names):
+            self._undo_stack.push(NewMapping(self._mappings_table_model, mapping, name))
+        self._undo_stack.endMacro()
+
+    @Slot(bool)
+    def _duplicate_mappings(self, checked):
+        """Duplicates selected mappings.
+
+        Args:
+            checked (bool): unused
+        """
+        selection_model = self._ui.mappings_table.selectionModel()
+        indices = [self._sort_mappings_table_model.mapToSource(i) for i in selection_model.selectedIndexes()]
+        if not indices:
+            return
+        specifications = []
+        names = []
+        for index in indices:
+            if index.column() != 0:
+                continue
+            specification = self._mappings_table_model.data(index, MappingsTableModel.MAPPING_SPECIFICATION_ROLE)
+            specifications.append(deepcopy(specification))
+            names.append(self._mappings_table_model.data(index))
+        text = "duplicate mapping" + ("s" if len(specifications) > 1 else "")
+        self._undo_stack.beginMacro(text)
+        for name, specification in zip(names, specifications):
+            self._undo_stack.push(NewMapping(self._mappings_table_model, specification, name))
+        self._undo_stack.endMacro()
+
     @Slot()
     def _toggle_all_enabled(self):
         """Pushes a command that enables or disables all mappings to undo stack."""
@@ -542,6 +687,11 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
                 mapping_type = MappingType.relationship_parameter_values
             else:
                 mapping_type = MappingType.relationship_parameter_default_values
+        elif type_label == "Relationship class with object parameter":
+            if parameter_type_label == "Default value":
+                mapping_type = MappingType.relationship_object_parameter_default_values
+            else:
+                mapping_type = MappingType.relationship_object_parameter_values
         elif type_label == "Object group":
             if parameter_type_label == "None":
                 mapping_type = MappingType.object_groups
@@ -561,6 +711,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         mapping_specification = _new_mapping_specification(mapping_type)
         if self._ui.fix_table_name_check_box.isChecked():
             mapping_specification.root = _add_fixed_table_name(mapping_specification.root)
+            mapping_specification.root.value = self._ui.fix_table_name_line_edit.text()
         self._undo_stack.beginMacro("change mapping type")
         self._undo_stack.push(SetMappingType(index, mapping_type))
         self._undo_stack.push(SetMapping(index, mapping_specification.root))
@@ -632,6 +783,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         if self._mappings_table_model.data(index, MappingsTableModel.MAPPING_TYPE_ROLE) in (
             MappingType.object_parameter_values,
             MappingType.relationship_parameter_values,
+            MappingType.relationship_object_parameter_values,
         ):
             set_parameter_dimensions(modified, dimensions)
         else:
@@ -681,7 +833,8 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
             return
         new_name = self._ui.fix_table_name_line_edit.text()
         old_name = index.data(MappingsTableModel.FIXED_TABLE_NAME_ROLE)
-        self._undo_stack.push(SetFixedTableName(index, old_name, new_name))
+        if new_name != old_name:
+            self._undo_stack.push(SetFixedTableName(index, old_name, new_name))
 
     def _set_use_fixed_table_name_flag_silently(self, flag):
         """
@@ -721,25 +874,44 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         index = self._sort_mappings_table_model.mapToSource(self._ui.mappings_table.currentIndex())
         if not index.isValid():
             return
-        self._undo_stack.beginMacro("change mapping's group function")
-        mapping = index.data(MappingsTableModel.MAPPING_ROOT_ROLE)
-        mapping = deepcopy(mapping)
-        mapping.group_fn = group_function_name_from_display(text)
-        self._undo_stack.push(SetMapping(index, mapping))
-        self._undo_stack.endMacro()
+        old = index.data(MappingsTableModel.GROUP_FN_ROLE)
+        self._undo_stack.push(SetGroupFunction(index, old, group_function_name_from_display(text)))
 
-    def _set_root_mapping_group_fn_silently(self, group_fn):
+    def _set_group_fn_silently(self, group_fn_display_name):
         """
         Sets group function in combo box without emitting signals.
 
         Args:
-            group_fn (str): group function name
+            group_fn_display_name (str): group function's display name
         """
-        if group_fn != self._ui.group_fn_combo_box.currentText():
+        if group_fn_display_name != self._ui.group_fn_combo_box.currentText():
             self._ui.group_fn_combo_box.currentTextChanged.disconnect(self._change_root_mapping_group_fn)
-            self._ui.group_fn_combo_box.setCurrentText(group_fn)
+            self._ui.group_fn_combo_box.setCurrentText(group_fn_display_name)
             self._ui.group_fn_combo_box.currentTextChanged.connect(self._change_root_mapping_group_fn)
 
+    @Slot(int)
+    def _change_highlight_dimension(self, dimension):
+        """Pushes a command to change highlight dimension to undo stack.
+
+        Args:
+            dimension (int): highlight dimension
+        """
+        index = self._sort_mappings_table_model.mapToSource(self._ui.mappings_table.currentIndex())
+        if not index.isValid():
+            return
+        old = index.data(MappingsTableModel.HIGHLIGHT_DIMENSION_ROLE)
+        self._undo_stack.push(SetHighlightDimension(index, old, dimension - 1))
+
+    def _set_highlight_dimension_silently(self, highlight_dimension):
+        """Sets highlight dimension without emitting signals.
+
+        Args:
+            highlight_dimension (int): highlight dimension
+        """
+        self._ui.highlight_dimension_spin_box.valueChanged.disconnect(self._change_highlight_dimension)
+        self._ui.highlight_dimension_spin_box.setValue(highlight_dimension + 1)
+        self._ui.highlight_dimension_spin_box.valueChanged.connect(self._change_highlight_dimension)
+
     @Slot(int)
     def _change_relationship_dimensions(self, dimensions):
         """
@@ -752,7 +924,12 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         mapping = self._mappings_table_model.data(index, MappingsTableModel.MAPPING_ROOT_ROLE)
         modified = deepcopy(mapping)
         set_relationship_dimensions(modified, dimensions)
+        current_mapping_type = self._ui.item_type_combo_box.currentText()
         self._undo_stack.beginMacro("change relationship dimensions")
+        if current_mapping_type == "Relationship class with object parameter":
+            highlight_dimension = self._mappings_table_model.data(index, MappingsTableModel.HIGHLIGHT_DIMENSION_ROLE)
+            if highlight_dimension >= dimensions:
+                self._undo_stack.push(SetHighlightDimension(index, highlight_dimension, dimensions))
         self._undo_stack.push(SetMapping(self._ui.mappings_table.currentIndex(), modified))
         self._undo_stack.endMacro()
 
@@ -766,6 +943,7 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
         self._ui.relationship_dimensions_spin_box.valueChanged.disconnect(self._change_relationship_dimensions)
         self._ui.relationship_dimensions_spin_box.setValue(dimensions)
         self._ui.relationship_dimensions_spin_box.valueChanged.connect(self._change_relationship_dimensions)
+        self._ui.highlight_dimension_spin_box.setMaximum(dimensions if dimensions > 0 else 1)
 
     def _enable_mapping_specification_editing(self):
         """Enables and disables mapping specification editing controls."""
@@ -776,21 +954,36 @@ class SpecificationEditorWindow(SpecificationEditorWindowBase):
 
     def _enable_relationship_controls(self):
         """Enables and disables controls related to relationship export."""
-        self._ui.relationship_dimensions_spin_box.setEnabled(
-            self._ui.item_type_combo_box.currentText() == "Relationship class"
+        current_mapping_type = self._ui.item_type_combo_box.currentText()
+        self._ui.relationship_dimensions_spin_box.setEnabled(current_mapping_type.startswith("Relationship class"))
+        self._ui.highlight_dimension_spin_box.setEnabled(
+            current_mapping_type == "Relationship class with object parameter"
         )
 
     def _enable_parameter_controls(self):
         """Enables and disables controls related to relationship export."""
         mapping_type = self._ui.item_type_combo_box.currentText()
-        if mapping_type in ("Object class", "Object group", "Relationship class"):
+        types_with_parameters = {
+            "Object class",
+            "Object group",
+            "Relationship class",
+            "Relationship class with object parameter",
+        }
+        if mapping_type in types_with_parameters:
             self._ui.parameter_type_combo_box.setEnabled(True)
+            self._ui.parameter_type_combo_box.currentTextChanged.disconnect(self._change_mapping_type)
             model = self._ui.parameter_type_combo_box.model()
             default_value_item = model.item(1)
             if mapping_type == "Object group":
                 default_value_item.setFlags(default_value_item.flags() & ~Qt.ItemIsEnabled)
             else:
                 default_value_item.setFlags(default_value_item.flags() | Qt.ItemIsEnabled)
+            no_value_item = model.item(2)
+            if mapping_type == "Relationship class with object parameter":
+                no_value_item.setFlags(default_value_item.flags() & ~Qt.ItemIsEnabled)
+            else:
+                no_value_item.setFlags(default_value_item.flags() | Qt.ItemIsEnabled)
+            self._ui.parameter_type_combo_box.currentTextChanged.connect(self._change_mapping_type)
             self._ui.parameter_dimensions_spin_box.setEnabled(self._ui.parameter_type_combo_box.currentText() != "None")
         else:
             self._ui.parameter_type_combo_box.setEnabled(False)
@@ -823,38 +1016,48 @@ def _new_mapping_specification(mapping_type):
         MappingSpecification: an export mapping specification
     """
     if mapping_type == MappingType.objects:
-        return MappingSpecification(mapping_type, True, True, False, object_export(0, 1))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, object_export(0, 1))
     if mapping_type == MappingType.object_groups:
-        return MappingSpecification(mapping_type, True, True, False, object_group_export(0, 1, 2))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, object_group_export(0, 1, 2))
     if mapping_type == MappingType.object_group_parameter_values:
         return MappingSpecification(
             mapping_type,
             True,
             True,
+            NoGroup.NAME,
             False,
             object_group_parameter_export(0, 1, Position.hidden, 2, 3, 4, Position.hidden, 5, None, None),
         )
     if mapping_type == MappingType.object_parameter_default_values:
         return MappingSpecification(
-            mapping_type, True, True, False, object_parameter_default_value_export(0, 1, Position.hidden, 2, None, None)
+            mapping_type,
+            True,
+            True,
+            NoGroup.NAME,
+            False,
+            object_parameter_default_value_export(0, 1, Position.hidden, 2, None, None),
         )
     if mapping_type == MappingType.object_parameter_values:
         return MappingSpecification(
             mapping_type,
             True,
             True,
+            NoGroup.NAME,
             False,
             object_parameter_export(0, 2, Position.hidden, 1, 3, Position.hidden, 4, None, None),
         )
     if mapping_type == MappingType.parameter_value_lists:
-        return MappingSpecification(mapping_type, True, True, False, parameter_value_list_export(0, 1))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, parameter_value_list_export(0, 1))
     if mapping_type == MappingType.relationships:
-        return MappingSpecification(mapping_type, True, True, False, relationship_export(0, Position.hidden, [1], [2]))
+        return MappingSpecification(
+            mapping_type, True, True, NoGroup.NAME, False, relationship_export(0, Position.hidden, [1], [2])
+        )
     if mapping_type == MappingType.relationship_parameter_default_values:
         return MappingSpecification(
             mapping_type,
             True,
             True,
+            NoGroup.NAME,
             False,
             relationship_parameter_default_value_export(0, 1, Position.hidden, 2, None, None),
         )
@@ -863,25 +1066,48 @@ def _new_mapping_specification(mapping_type):
             mapping_type,
             True,
             True,
+            NoGroup.NAME,
             False,
             relationship_parameter_export(
                 0, 3, Position.hidden, Position.hidden, [1], [2], 4, Position.hidden, 5, None, None
             ),
         )
+    if mapping_type == MappingType.relationship_object_parameter_default_values:
+        return MappingSpecification(
+            mapping_type,
+            True,
+            True,
+            NoGroup.NAME,
+            False,
+            relationship_object_parameter_default_value_export(0, 2, [1], Position.hidden, 3, None, None, 0),
+        )
+    if mapping_type == MappingType.relationship_object_parameter_values:
+        return MappingSpecification(
+            mapping_type,
+            True,
+            True,
+            NoGroup.NAME,
+            False,
+            relationship_object_parameter_export(
+                0, 3, Position.hidden, Position.hidden, [1], [2], 4, Position.hidden, 5, None, None, 0
+            ),
+        )
     if mapping_type == MappingType.alternatives:
-        return MappingSpecification(mapping_type, True, True, False, alternative_export(0))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, alternative_export(0))
     if mapping_type == MappingType.scenario_alternatives:
-        return MappingSpecification(mapping_type, True, True, False, scenario_alternative_export(0, 1))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, scenario_alternative_export(0, 1))
     if mapping_type == MappingType.scenarios:
-        return MappingSpecification(mapping_type, True, True, False, scenario_export(0, 1))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, scenario_export(0, 1))
     if mapping_type == MappingType.features:
-        return MappingSpecification(mapping_type, True, True, False, feature_export(0, 1))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, feature_export(0, 1))
     if mapping_type == MappingType.tools:
-        return MappingSpecification(mapping_type, True, True, False, tool_export(0))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, tool_export(0))
     if mapping_type == MappingType.tool_features:
-        return MappingSpecification(mapping_type, True, True, False, tool_feature_export(0, 1, 2, 3))
+        return MappingSpecification(mapping_type, True, True, NoGroup.NAME, False, tool_feature_export(0, 1, 2, 3))
     if mapping_type == MappingType.tool_feature_methods:
-        return MappingSpecification(mapping_type, True, True, False, tool_feature_method_export(0, 1, 2, 3))
+        return MappingSpecification(
+            mapping_type, True, True, NoGroup.NAME, False, tool_feature_method_export(0, 1, 2, 3)
+        )
     raise NotImplementedError()
 
 
@@ -913,4 +1139,4 @@ def _remove_fixed_table_name(mapping_root):
     Returns:
         Mapping: new mapping hierarchy
     """
-    return mapping_root.child
+    return unflatten(mapping_root.flatten()[1:])
