@@ -176,7 +176,10 @@ class PythonToolSpecOptionalWidget(OptionalWidget):
         p = self.ui.lineEdit_python_path.text()  # This may be an empty string
         j = ""
         item = self.kernel_spec_model.item(self.ui.comboBox_kernel_specs.currentIndex())
-        selected_kernel_spec = item.data()["kernel_spec_name"]
+        if not item.data():
+            selected_kernel_spec = ""
+        else:
+            selected_kernel_spec = item.data()["kernel_spec_name"]
         self._kernel_spec_editor = KernelEditor(self._parent, p, j, "python", selected_kernel_spec)
         self._kernel_spec_editor.finished.connect(self.python_kernel_editor_closed)
         self._kernel_spec_editor.open()
@@ -185,10 +188,12 @@ class PythonToolSpecOptionalWidget(OptionalWidget):
     def python_kernel_editor_closed(self, ret_code):
         """Catches the selected Python kernel name when the editor is closed."""
         item = self.kernel_spec_model.item(self.ui.comboBox_kernel_specs.currentIndex())
-        previous_kernel_spec = item.data()["kernel_spec_name"]
+        if not item.data():
+            previous_kernel_spec = ""
+        else:
+            previous_kernel_spec = item.data()["kernel_spec_name"]
         self._refresh_kernel_spec_model()
         if ret_code != 1:  # Editor closed with Cancel
-            # TODO: Is this needed?
             # Set previous kernel selected in Python kernel combobox if it still exists
             r = self.find_index_by_data(previous_kernel_spec)
             if r == -1:
